using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace UWB_GUI
{
    public class UWB_UI_CheckBox : UWB_UI_Element
    {
        private static readonly String CONTENT_CHECKBOX_PATH_NAME = "Images/CheckBox";
        private static readonly int CHECKBOX_TO_LABEL_SPACING = 3;
        private static readonly int CHECKBOX_WIDTH = 13;
        private static readonly int CHECKBOX_HEIGHT = 13;

        protected Texture2D m_Image;
        protected bool m_Selected;
        protected bool m_Highlighted;
        protected Rectangle m_SourceImageRect;
        protected Rectangle m_DrawArea;
        protected Color m_TintColor;
        protected Color m_FontColor;
        protected UWB_UI_Container m_Container;
        protected String m_Label;
        protected SpriteFont m_SpriteFont;
        protected Vector2 m_FontPosition;

        public UWB_UI_CheckBox(String label, UWB_UI_Container container, ContentManager content, SpriteFont font)
        {
            Initialize(label, container, content, font);
            m_DrawArea = new Rectangle(m_Container.X, m_Container.Y, CHECKBOX_WIDTH, CHECKBOX_HEIGHT);
            m_Bounds = new Rectangle(m_Container.X, m_Container.Y, CHECKBOX_WIDTH + CHECKBOX_TO_LABEL_SPACING + (int)font.MeasureString(label).X, getHeight());
        }

        public UWB_UI_CheckBox(String label, Vector2 position, UWB_UI_Container container, ContentManager content, SpriteFont font)
        {
            Initialize(label, container, content, font);
            m_DrawArea = new Rectangle((int)position.X + m_Container.X, (int)position.Y + m_Container.Y, CHECKBOX_WIDTH, CHECKBOX_HEIGHT);
            m_Bounds = new Rectangle((int)position.X + m_Container.X, (int)position.Y + m_Container.Y, CHECKBOX_WIDTH + CHECKBOX_TO_LABEL_SPACING + (int)font.MeasureString(label).X, getHeight());
        }

        private void Initialize(String label, UWB_UI_Container container, ContentManager content, SpriteFont font)
        {
            m_Image = content.Load<Texture2D>(CONTENT_CHECKBOX_PATH_NAME);
            m_Selected = false;
            m_Highlighted = false;
            m_TintColor = Color.White;
            m_FontColor = Color.Black;
            m_Container = container;
            m_Label = label;
            m_SpriteFont = font;
            m_FontPosition = new Vector2(0, 0);
            m_SourceImageRect = new Rectangle(0, 0, CHECKBOX_WIDTH, CHECKBOX_HEIGHT);
        }

        public virtual bool IsSelected
        {
            get { return m_Selected; }
            set { m_Selected = value; }
        }

        public virtual bool IsHighlighted
        {
            get { return m_Highlighted; }
            set { m_Highlighted = value; }
        }

        public virtual Color TintColor
        {
            get { return m_TintColor; }
            set { m_TintColor = value; }
        }

        public virtual Color FontColor
        {
            get { return m_FontColor; }
            set { m_FontColor = value; }
        }

        public virtual void setPosition(Vector2 position)
        {
            m_DrawArea.X = (int)position.X;
            m_DrawArea.Y = (int)position.Y;
        }

        public virtual int getHeight()
        {
            return Math.Max(m_SpriteFont.LineSpacing, m_DrawArea.Height);
        }

        public override void setHighlighted(Point mousePosition)
        {
            m_Highlighted = m_Bounds.Contains(mousePosition);
        }

        public override void setSelected(Point mousePosition)
        {
            if (m_Bounds.Contains(mousePosition))
            {
                m_Selected = !m_Selected;
            }
        }

        public override void Update(GameTime gameTime)
        {
            
        }

        public override void Draw(GraphicsDeviceManager graphics, SpriteBatch spriteBatch)
        {
            int horizontalOffset = 0;
            if (m_Highlighted)
                horizontalOffset = CHECKBOX_WIDTH;

            if (m_Selected)
                horizontalOffset += CHECKBOX_WIDTH * 2;

            m_SourceImageRect.X = horizontalOffset;

            m_FontPosition.X = m_DrawArea.X + CHECKBOX_WIDTH + CHECKBOX_TO_LABEL_SPACING;
            int verticaleOffset = m_SpriteFont.LineSpacing - m_DrawArea.Height;
            m_FontPosition.Y = m_DrawArea.Y - (verticaleOffset / 2);

            spriteBatch.Draw(m_Image, m_DrawArea, m_SourceImageRect, m_TintColor);
            spriteBatch.DrawString(m_SpriteFont, m_Label, m_FontPosition, m_FontColor);
        }
    }
}
