using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.Content;

namespace UWB_GUI
{
    public class UWB_UI_Container : UWB_UI_Element
    {
        private Viewport m_View;
        private Color m_ClearColor;
        private List<UWB_UI_Item> m_Items;

        public UWB_UI_Container(int x, int y, int width, int height, Color clearColor)
        {
            m_View = new Viewport();
            m_View.X = x;
            m_View.Y = y;
            m_View.Height = height;
            m_View.Width = width;
            m_Bounds = new Rectangle(x, y, width, height);
            m_Items = new List<UWB_UI_Item>();
            m_ClearColor = clearColor;
        }

        public void AddItem(UWB_UI_Item item)
        {
            m_Items.Add(item);
        }

        public void AddItem(String name, UWB_UI_Element item)
        {
            m_Items.Add(new UWB_UI_Item(name, item));
        }

        public UWB_UI_Element getItem(String name)
        {
            foreach (UWB_UI_Item item in m_Items)
            {
                if (item.Name == name)
                    return item.Item;
            }
            return null;
        }

        public void setSelectedRadioButton(UWB_UI_RadioButton radioButton)
        {
            foreach (UWB_UI_Item item in m_Items)
            {
                if (item.Item is UWB_UI_RadioButton)
                    ((UWB_UI_RadioButton)item.Item).IsSelected = false;
            }
            radioButton.IsSelected = true;
        }

        public int X
        {
            get { return m_View.X; }
            set { m_View.X = value; }
        }

        public int Y
        {
            get { return m_View.Y; }
            set { m_View.Y = value; }
        }

        public int Width
        {
            get { return m_View.Width; }
            set { m_View.Width = value; }
        }

        public int Height
        {
            get { return m_View.Height; }
            set { m_View.Height = value; }
        }

        public override void setHighlighted(Point mousePosition)
        {
            foreach (UWB_UI_Item item in m_Items)
            {
                item.Item.setHighlighted(mousePosition);
            }
        }

        public override void setSelected(Point mousePosition)
        {
            foreach (UWB_UI_Item item in m_Items)
            {
                item.Item.setSelected(mousePosition);
            }
        }

        public override void Update(GameTime gameTime)
        {
            foreach (UWB_UI_Item item in m_Items)
            {
                item.Update(gameTime);
            }
        }

        public override void Draw(GraphicsDeviceManager graphics, SpriteBatch spriteBatch)
        {
            Viewport tempView = graphics.GraphicsDevice.Viewport;

            graphics.GraphicsDevice.Viewport = m_View;
            graphics.GraphicsDevice.Clear(m_ClearColor);

            foreach (UWB_UI_Item item in m_Items)
            {
                item.Draw(graphics, spriteBatch);
            }

            graphics.GraphicsDevice.Viewport = tempView;
        }
    }
}
