using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using Microsoft.Xna.Framework.Content;

namespace UWB_GUI
{
    public class UWB_UI_Mouse : UWB_UI_Element
    {
        private static readonly String CONTENT_MOUSE_PATH_NAME = "Images/MousePointer";
        
        private Texture2D m_MouseImage;
        private Vector2 m_Position;
        private UWB_UI_Container m_ParentContainer;
        private Rectangle m_DrawArea;
        private Color m_Color;
        private float m_Sensitivity;
        private bool m_HasMouseMoved;

        public UWB_UI_Mouse(UWB_UI_Container parentContainer, ContentManager content)
        {
            m_MouseImage = content.Load<Texture2D>(CONTENT_MOUSE_PATH_NAME);
            m_Position = new Vector2(parentContainer.X, parentContainer.Y);
            m_Color = Color.White;
            m_Sensitivity = 1f;
            m_ParentContainer = parentContainer;
            m_DrawArea = new Rectangle(0, 0, m_MouseImage.Width, m_MouseImage.Height);
            m_Bounds = new Rectangle(0, 0, m_MouseImage.Width, m_MouseImage.Height);
            m_HasMouseMoved = false;
        }

        public void MoveMousePosition(Vector2 delta)
        {
            int temp = m_DrawArea.Left;
            float tempX = m_Position.X + (delta.X * m_Sensitivity);
            if (tempX <= (m_ParentContainer.Width + m_ParentContainer.X) &&
                tempX >= m_ParentContainer.X)
            {
                m_Position += new Vector2(delta.X * m_Sensitivity, 0);
            }

            float tempY = m_Position.Y + (delta.Y * m_Sensitivity);
            if (tempY <= (m_ParentContainer.Height + m_ParentContainer.Y) &&
                tempY >= m_ParentContainer.Y)
            {
                m_Position += new Vector2(0, delta.Y * m_Sensitivity);
            }
            m_HasMouseMoved = true;
        }

        public bool HasMouseMoved
        {
            get { return m_HasMouseMoved; }
            set { m_HasMouseMoved = value; }
        }

        public Vector2 Position
        {
            get { return m_Position; }
        }

        public Color TintColor
        {
            get { return m_Color; }
            set { m_Color = value; }
        }

        public float Sensitivity
        {
            get { return m_Sensitivity; }
            set { m_Sensitivity = value; }
        }

        public override Rectangle GetBounds()
        {
            m_Bounds.X = (int)m_Position.X;
            m_Bounds.Y = (int)m_Position.Y;
            return m_Bounds;
        }

        public override void Update(GameTime gameTime)
        {
        }

        public override void Draw(GraphicsDeviceManager graphics, SpriteBatch spriteBatch)
        {
            m_DrawArea.X = (int)m_Position.X;
            m_DrawArea.Y = (int)m_Position.Y;
            spriteBatch.Draw(m_MouseImage, m_DrawArea, m_Color);
        }
    }
}
