using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace UWB_GUI
{
    public class UWB_UI_RadioButton : UWB_UI_CheckBox
    {
        private static readonly String CONTENT_RADIOBUTTON_PATH_NAME = "Images/RadioButton";
        private static readonly int RADIOBUTTON_TO_LABEL_SPACING = 3;
        private static readonly int RADIOBUTTON_WIDTH = 12;
        private static readonly int RADIOBUTTON_HEIGHT = 12;

        public UWB_UI_RadioButton(String label, UWB_UI_Container container, ContentManager content, SpriteFont font) 
            : base(label, container, content, font)
        {
            m_Image = content.Load<Texture2D>(CONTENT_RADIOBUTTON_PATH_NAME);
            m_DrawArea = new Rectangle(m_Container.X, m_Container.Y, RADIOBUTTON_WIDTH, RADIOBUTTON_HEIGHT);
            m_Bounds = new Rectangle(m_Container.X, m_Container.Y, RADIOBUTTON_WIDTH + RADIOBUTTON_TO_LABEL_SPACING + (int)font.MeasureString(label).X, getHeight());
            m_SourceImageRect = new Rectangle(0, 0, RADIOBUTTON_WIDTH, RADIOBUTTON_HEIGHT);
        }

        public UWB_UI_RadioButton(String label, Vector2 position, UWB_UI_Container container, ContentManager content, SpriteFont font)
            : base(label, position, container, content, font)
        {
            m_Image = content.Load<Texture2D>(CONTENT_RADIOBUTTON_PATH_NAME);
            m_DrawArea = new Rectangle((int)position.X + m_Container.X, (int)position.Y + m_Container.Y, RADIOBUTTON_WIDTH, RADIOBUTTON_HEIGHT);
            m_Bounds = new Rectangle((int)position.X + m_Container.X, (int)position.Y + m_Container.Y, RADIOBUTTON_WIDTH + RADIOBUTTON_TO_LABEL_SPACING + (int)font.MeasureString(label).X, getHeight());
            m_SourceImageRect = new Rectangle(0, 0, RADIOBUTTON_WIDTH, RADIOBUTTON_HEIGHT);
        }

        public override void setHighlighted(Point mousePosition)
        {
            base.setHighlighted(mousePosition);
        }

        public override void setSelected(Point mousePosition)
        {
            if (m_Bounds.Contains(mousePosition))
            {
                m_Container.setSelectedRadioButton(this);
            }
        }

        public override void Update(Microsoft.Xna.Framework.GameTime gameTime)
        {
            
        }

        public override void Draw(Microsoft.Xna.Framework.GraphicsDeviceManager graphics, Microsoft.Xna.Framework.Graphics.SpriteBatch spriteBatch)
        {
            int horizontalOffset = 0;
            if (m_Highlighted)
                horizontalOffset = RADIOBUTTON_WIDTH;

            if (m_Selected)
                horizontalOffset += RADIOBUTTON_WIDTH * 2;

            m_SourceImageRect.X = horizontalOffset;

            m_FontPosition.X = m_DrawArea.X + RADIOBUTTON_WIDTH + RADIOBUTTON_TO_LABEL_SPACING;
            int verticaleOffset = m_SpriteFont.LineSpacing - m_DrawArea.Height;
            m_FontPosition.Y = m_DrawArea.Y - (verticaleOffset / 2);

            spriteBatch.Draw(m_Image, m_DrawArea, m_SourceImageRect, m_TintColor);
            spriteBatch.DrawString(m_SpriteFont, m_Label, m_FontPosition, m_FontColor);
        }
    }
}
