#region Using Statements
using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

using UWB_GUI;
#endregion

namespace XNAGUI
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        ContentManager content;

        private const int WINDOW_WIDTH = 800;
        private const int WINDOW_HEIGHT = 600;

        private Viewport m_DefaultViewPort;

        private SpriteBatch spriteBatch;
        UWB_UI_Container m_GUI;
        UWB_UI_Mouse m_Mouse;
        bool m_ShowGUI;

        private KeyboardState m_OldKeyState;

        private SpriteFont m_Arial8;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            //very important to make the back buffer dimension the same as the actual window dimensions
            graphics.PreferredBackBufferHeight = WINDOW_HEIGHT;
            graphics.PreferredBackBufferWidth = WINDOW_WIDTH;
            content = new ContentManager(Services);
            this.IsMouseVisible = true;
        }


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            base.Initialize();

            m_ShowGUI = true;
            m_GUI = new UWB_UI_Container(5, 60, WINDOW_WIDTH - 10, WINDOW_HEIGHT - 65, Color.Azure);
            UWB_UI_Container tempContainer = new UWB_UI_Container(10, 200, WINDOW_WIDTH - 20, WINDOW_HEIGHT - 210, Color.SlateGray);
            UWB_UI_Container tempContainer1 = new UWB_UI_Container(50, 300, 400, 100, Color.Silver);

            for (int i = 0; i < 5; i++)
            {
                m_GUI.AddItem(new UWB_UI_Item("CheckBox" + i.ToString(), new UWB_UI_CheckBox("Container 1: Check Box " + i.ToString(), new Vector2(5, 5 + (i * (m_Arial8.LineSpacing + 3))), m_GUI, content, m_Arial8)));
                tempContainer.AddItem(new UWB_UI_Item("CheckBox" + i.ToString(), new UWB_UI_CheckBox("Container 2: Check Box " + i.ToString(), new Vector2(5, 5 + (i * (m_Arial8.LineSpacing + 3))), tempContainer, content, m_Arial8)));
                tempContainer1.AddItem(new UWB_UI_Item("CheckBox" + i.ToString(), new UWB_UI_CheckBox("Container 3: Check Box " + i.ToString(), new Vector2(5, 5 + (i * (m_Arial8.LineSpacing + 3))), tempContainer1, content, m_Arial8))); 
            }

            for (int i = 0; i < 5; i++)
            {
                m_GUI.AddItem(new UWB_UI_Item("RadioButton" + i.ToString(), new UWB_UI_RadioButton("Container 1: Radio Button " + i.ToString(), new Vector2(200, 5 + (i * (m_Arial8.LineSpacing + 3))), m_GUI, content, m_Arial8)));
                tempContainer.AddItem(new UWB_UI_Item("RadioButton" + i.ToString(), new UWB_UI_RadioButton("Container 2: Radio Button " + i.ToString(), new Vector2(200, 5 + (i * (m_Arial8.LineSpacing + 3))), tempContainer, content, m_Arial8)));
                tempContainer1.AddItem(new UWB_UI_Item("RadioButton" + i.ToString(), new UWB_UI_RadioButton("Container 3: Radio Button " + i.ToString(), new Vector2(200, 5 + (i * (m_Arial8.LineSpacing + 3))), tempContainer1, content, m_Arial8)));
            }

            tempContainer.AddItem(new UWB_UI_Item("TempContainer1", tempContainer1));
            m_GUI.AddItem(new UWB_UI_Item("TempContainer", tempContainer));            
            m_Mouse = new UWB_UI_Mouse(m_GUI, content);

            m_DefaultViewPort = new Viewport();
            m_DefaultViewPort.X = 0;
            m_DefaultViewPort.Y = 0;
            m_DefaultViewPort.Width = WINDOW_WIDTH;
            m_DefaultViewPort.Height = WINDOW_HEIGHT;

            m_OldKeyState = new KeyboardState();

            spriteBatch = new SpriteBatch(graphics.GraphicsDevice);
        }


        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                m_Arial8 = content.Load<SpriteFont>("Font/Arial-8");
            }

            // TODO: Load any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent)
            {
                // TODO: Unload any ResourceManagementMode.Automatic content
                content.Unload();
            }

            // TODO: Unload any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            KeyboardState newKeyBoardState = Keyboard.GetState();

            if (Keyboard.GetState().IsKeyDown(Keys.Up))
            {
                m_Mouse.MoveMousePosition(-Vector2.UnitY);
            }

            if (Keyboard.GetState().IsKeyDown(Keys.Down))
            {
                m_Mouse.MoveMousePosition(Vector2.UnitY);
            }

            if (Keyboard.GetState().IsKeyDown(Keys.Right))
            {
                m_Mouse.MoveMousePosition(Vector2.UnitX);
            }

            if (Keyboard.GetState().IsKeyDown(Keys.Left))
            {
                m_Mouse.MoveMousePosition(-Vector2.UnitX);
            }

            if (m_Mouse.HasMouseMoved)
            {
                m_GUI.setHighlighted(new Point((int)m_Mouse.Position.X, (int)m_Mouse.Position.Y));
                m_Mouse.HasMouseMoved = false;
            }

            if (newKeyBoardState.IsKeyDown(Keys.Space))
            {

            }
            else if (m_OldKeyState.IsKeyDown(Keys.Space))
            {
                m_GUI.setSelected(new Point((int)m_Mouse.Position.X, (int)m_Mouse.Position.Y));
            }

            m_OldKeyState = newKeyBoardState;

            base.Update(gameTime);
        }


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Viewport = m_DefaultViewPort;
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            if (m_ShowGUI)
            {
                spriteBatch.Begin();
                m_GUI.Draw(graphics, spriteBatch);
                m_Mouse.Draw(graphics, spriteBatch);
                spriteBatch.DrawString(m_Arial8, CreateMousePositionString(), new Vector2(5, 5), Color.White);
                spriteBatch.DrawString(m_Arial8, CreateUWBMousePositionString(), new Vector2(5, 10 + m_Arial8.LineSpacing), Color.White);
                spriteBatch.End();
            }

            base.Draw(gameTime);
        }

        public String CreateMousePositionString()
        {
            MouseState mouseState = Mouse.GetState();
            StringBuilder sb = new StringBuilder();
            sb.Append("Mouse Position: (");
            sb.Append(mouseState.X);
            sb.Append(",");
            sb.Append(mouseState.Y);
            sb.Append(")");
            return sb.ToString();
        }

        public String CreateUWBMousePositionString()
        {
            Vector2 pos = m_Mouse.Position;
            StringBuilder sb = new StringBuilder();
            sb.Append("UWB Mouse Position: (");
            sb.Append(pos.X);
            sb.Append(",");
            sb.Append(pos.Y);
            sb.Append(")");
            return sb.ToString();
        }
    }
}
